import { Component, Prop, h, Host, State, Event } from '@stencil/core';
import creditCardType, { getTypeInfo } from 'credit-card-type';
import { CreditCardInputErrorCodes } from '../../utils/credit-card-input-errors';
export class MadPaymentCardInput {
  constructor() {
    // DEFAULTS
    this.cardTypes = ['visa', 'mastercard', 'american-express', 'diners-club', 'discover', 'jcb', 'maestro'];
    this.baseCardTypes = ['visa', 'mastercard', 'american-express', 'discover'];
    this.defaultMax = 24;
    /**
     * The possible card types based on the input
     */
    this.possibleCards = ['visa', 'mastercard', 'american-express', 'discover'];
    /**
     * The max size of the input field
    */
    this.max = this.defaultMax;
  }
  handleChange(event) {
    event.preventDefault();
    const errors = [];
    const val = event.target.value.replace(/ /g, '');
    const cardTypes = creditCardType(val);
    const allPossibleCards = cardTypes.filter(({ type }) => this.cardTypes.includes(type)).map(({ type }) => type);
    const updateEvent = {
      value: val,
      complete: false,
      cardType: allPossibleCards,
      valid: allPossibleCards.length !== 0,
    };
    this.possibleCards = allPossibleCards.filter((val) => this.baseCardTypes.includes(val));
    if (allPossibleCards.length === 1) {
      const { lengths: possibleLengths, type: cardType, code: { size: cvvSize } } = cardTypes[0];
      const [max] = possibleLengths.slice(-1);
      this.value = this.prettyCardNumber(val, cardType);
      this.max = max;
      updateEvent.complete = this.max === val.length;
      updateEvent.cvvMax = cvvSize;
    }
    else if (allPossibleCards.length === 0) {
      errors.push(CreditCardInputErrorCodes.UNKNOWN_BRAND);
    }
    else {
      errors.push(CreditCardInputErrorCodes.INCOMPLETE);
      this.max = this.defaultMax;
    }
    if (errors)
      updateEvent.errors = errors;
    this.update.emit(updateEvent);
  }
  handleKeyPress(e) {
    if (isNaN(e.key) || e.key.trim() === '' || this.isMax()) {
      e.preventDefault();
    }
  }
  isMax() {
    const val = this.value ? this.value.replace(/ /g, '') : '';
    return val.length === this.max;
  }
  prettyCardNumber(cardNumber, cardType) {
    var card = getTypeInfo(cardType);
    if (card) {
      var offsets = [].concat(0, card.gaps, cardNumber.length);
      var components = [];
      for (var i = 0; offsets[i] < cardNumber.length; i++) {
        var start = offsets[i];
        var end = Math.min(offsets[i + 1], cardNumber.length);
        components.push(cardNumber.substring(start, end));
      }
      return components.join(" ");
    }
    return cardNumber;
  }
  render() {
    return (h(Host, null,
      h("mad-payment-input", { label: this.label, onPayKeyPress: ({ detail: e }) => { this.handleKeyPress(e); }, onPayInput: ({ detail: e }) => { this.handleChange(e); }, value: this.value, placeholder: this.placeholder }, this.baseCardTypes.map((card) => {
        return h("mad-payment-icon", { icon: card, class: `card-icon ${this.possibleCards.includes(card) ? 'visible' : ''}`, slot: "icons" });
      }))));
  }
  static get is() { return "mad-payment-card-input"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["card-input.css"]
  }; }
  static get styleUrls() { return {
    "$": ["card-input.css"]
  }; }
  static get properties() { return {
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Input placeholder text"
      },
      "attribute": "placeholder",
      "reflect": false
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Label text is optional, if present label is dplayed"
      },
      "attribute": "label",
      "reflect": false
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Input name"
      },
      "attribute": "name",
      "reflect": false
    }
  }; }
  static get states() { return {
    "value": {},
    "possibleCards": {},
    "max": {}
  }; }
  static get events() { return [{
      "method": "update",
      "name": "update",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "When value updated a CreditCardUpdate event will be published"
      },
      "complexType": {
        "original": "CreditCardUpdate",
        "resolved": "CreditCardUpdate",
        "references": {
          "CreditCardUpdate": {
            "location": "import",
            "path": "../../model/CreditCardUpdate"
          }
        }
      }
    }]; }
}
